unit http;

interface

uses
    Classes, blcksock, winsock, Synautil, SysUtils;

type
  TTCPHttpDaemon = class(TThread)
  private
    Sock : TTCPBlockSocket;
    BindPort : String;
  public
    Constructor Create(Port : String); // select another port but 80
    Destructor Destroy; override;
    procedure Execute; override;
  end;

  TTCPHttpThrd = class(TThread)
  private
    Sock:TTCPBlockSocket;
  public
    Headers: TStringList;
    InputData, OutputData: TMemoryStream;
    Constructor Create (hsock:tSocket);
    Destructor Destroy; override;
    procedure Execute; override;
    function ProcessHttpRequest(Request, URI: string): integer;
  end;


implementation

uses main;    // THE owner of the whole stuff

{ TTCPHttpDaemon }

Constructor TTCPHttpDaemon.Create(Port : String);
begin
     sock := TTCPBlockSocket.create;
     FreeOnTerminate := true;
     Priority := tpNormal;
     BindPort := Port;
     inherited create(false);
end;

Destructor TTCPHttpDaemon.Destroy;
begin
  Sock.free;
  inherited Destroy;
end;

procedure TTCPHttpDaemon.Execute;
var
  ClientSock:TSocket;
begin
     with sock do
     begin
          CreateSocket;
          setLinger(true,10);
          bind('0.0.0.0', BindPort);
          listen;
          repeat
                if terminated then break;
                if canread(1000) then
                begin
                     ClientSock:=accept;
                     if lastError=0 then TTCPHttpThrd.create(ClientSock);
                end;
          until false;
    end;
end;

{ TTCPHttpThrd }

Constructor TTCPHttpThrd.Create(Hsock:TSocket);
begin
     sock := TTCPBlockSocket.create;
     Headers := TStringList.Create;
     InputData := TMemoryStream.Create;
     OutputData := TMemoryStream.Create;
     Sock.socket := HSock;
     FreeOnTerminate := true;
     Priority := tpNormal;
     inherited create(false);
end;

Destructor TTCPHttpThrd.Destroy;
begin
     Sock.free;
     Headers.Free;
     InputData.Free;
     OutputData.Free;
     inherited Destroy;
end;

procedure TTCPHttpThrd.Execute;
var
   b : byte;
   method, uri, protocol : string;
   timeout               : integer;
   resultcode            : integer;
   s                     : string;
   size                  : integer;
   x, n                  : integer;
begin
     timeout := 120000;
     //read request line
     s := sock.RecvString(timeout);
     if sock.lasterror <> 0 then Exit;
     if s = '' then Exit;
     method := fetch(s, ' ');
     if (s = '') or (method = '') then Exit;
     uri := fetch(s, ' ');
     if uri = '' then Exit;
     protocol := fetch(s, ' ');
     headers.Clear;
     size := -1;

     //read request headers
     if protocol <> '' then
     begin
          if pos('HTTP/', protocol) <> 1 then Exit;
          repeat
                s := sock.RecvString(Timeout);
                if sock.lasterror <> 0 then Exit;
                if s <> '' then Headers.add(s);
                if Pos('CONTENT-LENGTH:', Uppercase(s)) = 1 then
                   Size := StrToIntDef(SeparateRight(s, ' '), -1);
          until s = '';
     end;

     //recv document...
     InputData.Clear;
     if size >= 0 then
     begin
          InputData.SetSize(Size);
          x := Sock.RecvBufferEx(InputData.Memory, Size, Timeout);
          InputData.SetSize(x);
          if sock.lasterror <> 0 then Exit;
     end;
     OutputData.Clear;
     ResultCode := ProcessHttpRequest(method, uri);
     sock.SendString('HTTP/1.0 ' + IntTostr(ResultCode) + CRLF);
     if protocol <> '' then
     begin
          with headers do
          begin
               Add('Content-size: ' + IntTostr(OutputData.Size));
               Add('Connection: close');
               Add('Date: ' + Rfc822DateTime(now));
               Add('Server: Synapse HTTP server demo');
               Add('');
          end;
          for n := 0 to headers.count - 1 do
          begin
               sock.sendstring(headers[n] + CRLF);
          end;
     end;
     if sock.lasterror <> 0 then Exit;
     Sock.SendBuffer(OutputData.Memory, OutputData.Size);
end;

function TTCPHttpThrd.ProcessHttpRequest(Request, URI: string): integer;
var
   htmllines   : TStringlist;
begin
//sample of precessing HTTP request:
// InputData is uploaded document, headers is stringlist with request headers.
// Request is type of request and URI is URI of request
// OutputData is document with reply, headers is stringlist with reply headers.
// Result is result code
  result := 504;

  if (request = 'GET') AND false then
  begin
       headers.Clear;
       headers.Add('Content-type: Text/Html');

       htmllines := TStringList.Create;
       try
          with htmllines do
          begin
               Add('<html>');
               Add('<head></head>');
               Add('<body>');
               Add('Request Uri: ' + uri);
               Add('<br>');
               Add('This document is generated by Synapse HTTP server demo!');
               Add('</body>');
               Add('</html>');
               SaveToStream(OutputData);
          end;
       finally
              htmllines.free;
       end;
  end
  else if (request = 'GET') AND true then
  begin
       headers.Clear;
       headers.Add('Content-type: image/jpeg');
       Form1.GetPicture(OutputData);
  end;
       Result := 200;
end;




end.
